/*
 * example_stream.c
 *
 * demonstrates operation of libri using the data streaming interface
 * continously take unlimited amounts of data.
*/

// include libri
#include "ri.h"

// for printf
#include "stdio.h"
// for clock
#include "time.h"


// to stream data, a callback function of this form is used.  This function
// is passed as an argument to ri_start_continuous_transfer and is'
// called everytime new data is available. Data streaming continues
// until this function returns false.

// data is a pointer to the raw adc data returned by the device.  if data is to
// be saved it should be copied elsewhere as this buffer may be reused internally
// or may be deleted as soon as this function returns.

// ndata is the number of valid samples in data

// dataloss is non-zero if dataloss is detected,  typically by an unstable
// usb connection.

// userdata is the pointer passed to ri_start_continuous_transfer

int transfer_callback(uint16_t* data, int ndata, int dataloss, void *userdata)
{
    // we've passed a pointer to a 64 bit int in the main function to keep
    // track of how many samples are left to be collected.
    int64_t* samples_left = (int64_t*)userdata;

    if (dataloss)
        printf("data loss detected\n");

    // simply count the number of samples collected and stop data collection
    // when sufficient bytes have been acquired.
    *samples_left -= ndata;
    return *samples_left > 0;
}



int main(int argc, char *argv[])
{
    // variable to check the return status of libri functions.
    // functions return 0 on success and non zero when an error
    // occurs.
    int err = 0;

    // call this function once to initialize the library.
    // when finished, call ri_exit() to uninitialize the library.
    ri_init();

    // references to the DPD80 are stored as an opaque pointer with the type ri_device*
    ri_device* device = 0;

    // try and open the first available device.
    device = ri_open_device();
    if (device == 0) {
        printf("unable to find a ri device\n");
	}
	else {

		// record initial time to measure the data transfer speed
		//
		const int64_t samples_to_transfer = 80 * 1000 * 1000;
		double initial_time = (double)clock() / CLOCKS_PER_SEC;

		// create a variable to store the number of samples to be acquired.
		// a pointer will be passed to ri_start_continuous_transfer so it can be
		// modified in the transfer_callback function.
		int64_t samples_left = samples_to_transfer;

		//start the acquisition process
		printf("transferring %lld samples...\n", samples_to_transfer);
		ri_start_continuous_transfer(device, transfer_callback, &samples_left);

		// get final time and calculate collection rate
		double final_time = (double)clock() / CLOCKS_PER_SEC;
		double MBs_transferred = samples_to_transfer*2. / 1000000.;
		printf("transfered %.1f MB in %g s of cpu time\n", MBs_transferred, final_time - initial_time);
		printf("%.2f MBPS", MBs_transferred / (final_time - initial_time));


		// close the device;
		device = ri_close_device(device);
	}

    // uninitialize the library
    ri_exit();

#ifdef _MSC_VER
	// if Visual Studio get keyboard input at end of program so it doesn't automatically close
	getchar();
#endif
    return 0;
}
